@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-
+from __future__ import print_function
 from tlslite import HTTPTLSConnection, HandshakeSettings
 
 settings = HandshakeSettings()
@@ -8,4 +8,4 @@ settings.useExperimentalTackExtension = True
 h = HTTPTLSConnection("localhost", 4443, settings=settings)    
 h.request("GET", "/index.html")
 r = h.getresponse()
-print r.read()
+print(r.read())
