@@ -204,7 +204,7 @@ class CipherSuite:
                 suites.append(CipherSuite.TLS_DH_ANON_WITH_AES_128_CBC_SHA)
             elif cipher == "aes256":
                 suites.append(CipherSuite.TLS_DH_ANON_WITH_AES_256_CBC_SHA)
-            return suites   
+        return suites   
     
     tripleDESSuites = []
     tripleDESSuites.append(TLS_SRP_SHA_WITH_3DES_EDE_CBC_SHA)
@@ -234,22 +234,22 @@ class Fault:
     badUsername = 101
     badPassword = 102
     badA = 103
-    clientSrpFaults = range(101,104)
+    clientSrpFaults = list(range(101,104))
 
     badVerifyMessage = 601
-    clientCertFaults = range(601,602)
+    clientCertFaults = list(range(601,602))
 
     badPremasterPadding = 501
     shortPremasterSecret = 502
-    clientNoAuthFaults = range(501,503)
+    clientNoAuthFaults = list(range(501,503))
 
     badB = 201
-    serverFaults = range(201,202)
+    serverFaults = list(range(201,202))
 
     badFinished = 300
     badMAC = 301
     badPadding = 302
-    genericFaults = range(300,303)
+    genericFaults = list(range(300,303))
 
     faultAlerts = {\
         badUsername: (AlertDescription.unknown_psk_identity, \
