@@ -2,13 +2,18 @@
 #   Trevor Perrin
 #   Kees Bos - Added ignoreAbruptClose parameter
 #   Dimitris Moraitis - Anon ciphersuites
+#   Martin von Loewis - python 3 port
 #
 # See the LICENSE file for legal information regarding use of this file.
 
 """TLS Lite + httplib."""
 
 import socket
-import httplib
+try:
+    import httplib
+except ImportError:
+    # Python 3
+    from http import client as httplib
 from tlslite.tlsconnection import TLSConnection
 from tlslite.integration.clienthelper import ClientHelper
 
