@@ -1,8 +1,9 @@
 # Authors: 
 #   Trevor Perrin
 #   Google - added reqCAs parameter
-#   Google (adapted by Sam Rushing) - NPN support
+#   Google (adapted by Sam Rushing and Marcelo Fernandez) - NPN support
 #   Dimitris Moraitis - Anon ciphersuites
+#   Martin von Loewis - python 3 port
 #
 # See the LICENSE file for legal information regarding use of this file.
 
@@ -211,7 +212,8 @@ class TLSConnection(TLSRecordLayer):
 
     def handshakeClientCert(self, certChain=None, privateKey=None,
                             session=None, settings=None, checker=None,
-                            reqTack=True, serverName="", async=False):
+                            nextProtos=None, reqTack=True, serverName="",
+                            async=False):
         """Perform a certificate-based handshake in the role of client.
 
         This function performs an SSL or TLS handshake.  The server
@@ -257,6 +259,10 @@ class TLSConnection(TLSRecordLayer):
         invoked to examine the other party's authentication
         credentials, if the handshake completes succesfully.
         
+        @type nextProtos: list of strings.
+        @param nextProtos: A list of upper layer protocols ordered by
+        preference, to use in the Next-Protocol Negotiation Extension.
+        
         @type reqTack: bool
         @param reqTack: Whether or not to send a "tack" TLS Extension, 
         requesting the server return a TackExtension if it has one.        
@@ -286,7 +292,7 @@ class TLSConnection(TLSRecordLayer):
         handshaker = self._handshakeClientAsync(certParams=(certChain,
                         privateKey), session=session, settings=settings,
                         checker=checker, serverName=serverName, 
-                        reqTack=reqTack)
+                        nextProtos=nextProtos, reqTack=reqTack)
         # The handshaker is a Python Generator which executes the handshake.
         # It allows the handshake to be run in a "piecewise", asynchronous
         # fashion, returning 1 when it is waiting to able to write, 0 when
@@ -302,7 +308,7 @@ class TLSConnection(TLSRecordLayer):
 
     def _handshakeClientAsync(self, srpParams=(), certParams=(), anonParams=(),
                              session=None, settings=None, checker=None,
-                             serverName="", reqTack=True):
+                             nextProtos=None, serverName="", reqTack=True):
 
         handshaker = self._handshakeClientAsyncHelper(srpParams=srpParams,
                 certParams=certParams,
@@ -310,13 +316,14 @@ class TLSConnection(TLSRecordLayer):
                 session=session,
                 settings=settings,
                 serverName=serverName,
+                nextProtos=nextProtos,
                 reqTack=reqTack)
         for result in self._handshakeWrapperAsync(handshaker, checker):
             yield result
 
 
     def _handshakeClientAsyncHelper(self, srpParams, certParams, anonParams,
-                               session, settings, serverName, reqTack):
+                               session, settings, serverName, nextProtos, reqTack):
         
         self._handshakeStart(client=True)
 
@@ -353,6 +360,9 @@ class TLSConnection(TLSRecordLayer):
                 reqTack = False
             if not settings or not settings.useExperimentalTackExtension:
                 reqTack = False
+        if nextProtos is not None:
+            if len(nextProtos) == 0:
+                raise ValueError("Caller passed no nextProtos")
         
         # Validates the settings and filters out any unsupported ciphers
         # or crypto libraries that were requested        
@@ -395,7 +405,8 @@ class TLSConnection(TLSRecordLayer):
         # Send the ClientHello.
         for result in self._clientSendClientHello(settings, session, 
                                         srpUsername, srpParams, certParams,
-                                        anonParams, serverName, reqTack):
+                                        anonParams, serverName, nextProtos,
+                                        reqTack):
             if result in (0,1): yield result
             else: break
         clientHello = result
@@ -407,10 +418,15 @@ class TLSConnection(TLSRecordLayer):
         serverHello = result
         cipherSuite = serverHello.cipher_suite
         
+        # Choose a matching Next Protocol from server list against ours
+        # (string or None)
+        nextProto = self._clientSelectNextProto(nextProtos, serverHello)
+
         #If the server elected to resume the session, it is handled here.
         for result in self._clientResume(session, serverHello, 
                         clientHello.random, 
-                        settings.cipherImplementations):
+                        settings.cipherImplementations,
+                        nextProto):
             if result in (0,1): yield result
             else: break
         if result == "resumed_and_finished":
@@ -429,6 +445,7 @@ class TLSConnection(TLSRecordLayer):
                 if result in (0,1): yield result
                 else: break                
             (premasterSecret, serverCertChain, tackExt) = result
+
         #If the server selected an anonymous ciphersuite, the client
         #finishes reading the post-ServerHello messages.
         elif cipherSuite in CipherSuite.anonSuites:
@@ -460,7 +477,8 @@ class TLSConnection(TLSRecordLayer):
         for result in self._clientFinished(premasterSecret,
                             clientHello.random, 
                             serverHello.random,
-                            cipherSuite, settings.cipherImplementations):
+                            cipherSuite, settings.cipherImplementations,
+                            nextProto):
                 if result in (0,1): yield result
                 else: break
         masterSecret = result
@@ -469,13 +487,13 @@ class TLSConnection(TLSRecordLayer):
         self.session = Session()
         self.session.create(masterSecret, serverHello.session_id, cipherSuite,
             srpUsername, clientCertChain, serverCertChain,
-            tackExt, serverHello.tackExt!=None, clientHello.server_name)
+            tackExt, serverHello.tackExt!=None, serverName)
         self._handshakeDone(resumed=False)
 
 
     def _clientSendClientHello(self, settings, session, srpUsername,
                                 srpParams, certParams, anonParams, 
-                                serverName, reqTack):
+                                serverName, nextProtos, reqTack):
         #Initialize acceptable ciphersuites
         cipherSuites = [CipherSuite.TLS_EMPTY_RENEGOTIATION_INFO_SCSV]
         if srpParams:
@@ -501,16 +519,20 @@ class TLSConnection(TLSRecordLayer):
                 clientHello = ClientHello()
                 clientHello.create(settings.maxVersion, getRandomBytes(32),
                                    session.sessionID, cipherSuites,
-                                   certificateTypes, session.srpUsername,
-                                   reqTack, False, session.serverName)
+                                   certificateTypes, 
+                                   session.srpUsername,
+                                   reqTack, nextProtos is not None,
+                                   session.serverName)
 
         #Or send ClientHello (without)
         else:
             clientHello = ClientHello()
             clientHello.create(settings.maxVersion, getRandomBytes(32),
-                               createByteArraySequence([]), cipherSuites,
-                               certificateTypes, srpUsername,
-                               reqTack, False, serverName)
+                               bytearray(0), cipherSuites,
+                               certificateTypes, 
+                               srpUsername,
+                               reqTack, nextProtos is not None, 
+                               serverName)
         for result in self._sendMsg(clientHello):
             yield result
         yield clientHello
@@ -562,15 +584,39 @@ class TLSConnection(TLSRecordLayer):
                     AlertDescription.illegal_parameter,
                     "Server responded with unrequested Tack Extension"):
                     yield result
+        if serverHello.next_protos and not clientHello.supports_npn:
+            for result in self._sendError(\
+                AlertDescription.illegal_parameter,
+                "Server responded with unrequested NPN Extension"):
+                yield result
             if not serverHello.tackExt.verifySignatures():
                 for result in self._sendError(\
                     AlertDescription.decrypt_error,
                     "TackExtension contains an invalid signature"):
                     yield result
         yield serverHello
+
+    def _clientSelectNextProto(self, nextProtos, serverHello):
+        # nextProtos is None or non-empty list of strings
+        # serverHello.next_protos is None or possibly-empty list of strings
+        #
+        # !!! We assume the client may have specified nextProtos as a list of
+        # strings so we convert them to bytearrays (it's awkward to require
+        # the user to specify a list of bytearrays or "bytes", and in 
+        # Python 2.6 bytes() is just an alias for str() anyways...
+        if nextProtos is not None and serverHello.next_protos is not None:
+            for p in nextProtos:
+                if bytearray(p) in serverHello.next_protos:
+                    return bytearray(p)
+            else:
+                # If the client doesn't support any of server's protocols,
+                # or the server doesn't advertise any (next_protos == [])
+                # the client SHOULD select the first protocol it supports.
+                return bytearray(nextProtos[0])
+        return None
  
     def _clientResume(self, session, serverHello, clientRandom, 
-                        cipherImplementations):
+                      cipherImplementations, nextProto):
         #If the server agrees to resume
         if session and session.sessionID and \
             serverHello.session_id == session.sessionID:
@@ -590,7 +636,7 @@ class TLSConnection(TLSRecordLayer):
             #Exchange ChangeCipherSpec and Finished messages
             for result in self._getFinished(session.masterSecret):
                 yield result
-            for result in self._sendFinished(session.masterSecret):
+            for result in self._sendFinished(session.masterSecret, nextProto):
                 yield result
 
             #Set the session for this connection
@@ -689,7 +735,8 @@ class TLSConnection(TLSRecordLayer):
         A = powMod(g, a, N)
 
         #Calculate client's static DH values (x, v)
-        x = makeX(bytesToString(s), srpUsername, password)
+        x = makeX(s, bytearray(srpUsername, "utf-8"),
+                    bytearray(password, "utf-8"))
         v = powMod(g, x, N)
 
         #Calculate u
@@ -703,7 +750,7 @@ class TLSConnection(TLSRecordLayer):
             A = N
             S = 0
             
-        premasterSecret = numberToBytes(S)
+        premasterSecret = numberToByteArray(S)
 
         #Send ClientKeyExchange
         for result in self._sendMsg(\
@@ -809,11 +856,10 @@ class TLSConnection(TLSRecordLayer):
                                          premasterSecret,
                                          clientRandom,
                                          serverRandom)
-                verifyBytes = self._calcSSLHandshakeHash(masterSecret, "")
+                verifyBytes = self._calcSSLHandshakeHash(masterSecret, b"")
             elif self.version in ((3,1), (3,2)):
-                verifyBytes = stringToBytes(\
-                    self._handshake_md5.digest() + \
-                    self._handshake_sha.digest())
+                verifyBytes = self._handshake_md5.digest() + \
+                                self._handshake_sha.digest()
             if self.fault == Fault.badVerifyMessage:
                 verifyBytes[0] = ((verifyBytes[0]+1) % 256)
             signedBytes = privateKey.sign(verifyBytes)
@@ -851,12 +897,13 @@ class TLSConnection(TLSRecordLayer):
             
         #Calculate premaster secret
         S = powMod(dh_Ys, dh_Xc, dh_p)
-        premasterSecret = numberToBytes(S)
+        premasterSecret = numberToByteArray(S)
                      
         yield (premasterSecret, None, None)
         
     def _clientFinished(self, premasterSecret, clientRandom, serverRandom,
-                        cipherSuite, cipherImplementations):                   
+                        cipherSuite, cipherImplementations, nextProto):
+
         masterSecret = calcMasterSecret(self.version, premasterSecret,
                             clientRandom, serverRandom)
         self._calcPendingStates(cipherSuite, masterSecret, 
@@ -864,9 +911,9 @@ class TLSConnection(TLSRecordLayer):
                                 cipherImplementations)
 
         #Exchange ChangeCipherSpec and Finished messages
-        for result in self._sendFinished(masterSecret):
+        for result in self._sendFinished(masterSecret, nextProto):
             yield result
-        for result in self._getFinished(masterSecret):
+        for result in self._getFinished(masterSecret, nextProto=nextProto):
             yield result
         yield masterSecret
 
@@ -982,6 +1029,11 @@ class TLSConnection(TLSRecordLayer):
         will be sent along with a certificate request. This does not affect
         verification.        
 
+        @type nextProtos: list of strings.
+        @param nextProtos: A list of upper layer protocols to expose to the
+        clients through the Next-Protocol Negotiation Extension, 
+        if they support it.
+
         @raise socket.error: If a socket error occurs.
         @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
         without a preceding alert.
@@ -1044,7 +1096,9 @@ class TLSConnection(TLSRecordLayer):
             raise ValueError("Caller passed reqCAs but not reqCert")            
         if certChain and not isinstance(certChain, X509CertChain):
             raise ValueError("Unrecognized certificate type")
-        if (tacks or activationFlags):
+        if activationFlags and not tacks:
+            raise ValueError("Nonzero activationFlags requires tacks")
+        if tacks:
             if not tackpyLoaded:
                 raise ValueError("tackpy is not loaded")
             if not settings or not settings.useExperimentalTackExtension:
@@ -1074,7 +1128,7 @@ class TLSConnection(TLSRecordLayer):
         if sessionCache:
             sessionID = getRandomBytes(32)
         else:
-            sessionID = createByteArraySequence([])
+            sessionID = bytearray(0)
         
         if not clientHello.supports_npn:
             nextProtos = None
@@ -1139,13 +1193,19 @@ class TLSConnection(TLSRecordLayer):
             serverCertChain = certChain
         else:
             serverCertChain = None
+        srpUsername = None
+        serverName = None
+        if clientHello.srp_username:
+            srpUsername = clientHello.srp_username.decode("utf-8")
+        if clientHello.server_name:
+            serverName = clientHello.server_name.decode("utf-8")
         self.session.create(masterSecret, serverHello.session_id, cipherSuite,
-            clientHello.srp_username, clientCertChain, serverCertChain,
-            tackExt, serverHello.tackExt!=None, clientHello.server_name)
+            srpUsername, clientCertChain, serverCertChain,
+            tackExt, serverHello.tackExt!=None, serverName)
             
         #Add the session object to the session cache
         if sessionCache and sessionID:
-            sessionCache[bytesToString(sessionID)] = self.session
+            sessionCache[sessionID] = self.session
 
         self._handshakeDone(resumed=False)
 
@@ -1201,8 +1261,7 @@ class TLSConnection(TLSRecordLayer):
             #Check in the session cache
             if sessionCache and not session:
                 try:
-                    session = sessionCache[bytesToString(\
-                                               clientHello.session_id)]
+                    session = sessionCache[clientHello.session_id]
                     if not session.resumable:
                         raise AssertionError()
                     #Check for consistency with ClientHello
@@ -1215,12 +1274,14 @@ class TLSConnection(TLSRecordLayer):
                                 AlertDescription.handshake_failure):
                             yield result
                     if clientHello.srp_username:
-                        if clientHello.srp_username != session.srpUsername:
+                        if not session.srpUsername or \
+                            clientHello.srp_username != bytearray(session.srpUsername, "utf-8"):
                             for result in self._sendError(\
                                     AlertDescription.handshake_failure):
                                 yield result
                     if clientHello.server_name:
-                        if clientHello.server_name != session.serverName:
+                        if not session.serverName or \
+                            clientHello.server_name != bytearray(session.serverName, "utf-8"):
                             for result in self._sendError(\
                                     AlertDescription.handshake_failure):
                                 yield result                    
@@ -1297,10 +1358,11 @@ class TLSConnection(TLSRecordLayer):
     def _serverSRPKeyExchange(self, clientHello, serverHello, verifierDB, 
                                 cipherSuite, privateKey, serverCertChain):
 
-        self.allegedSrpUsername = clientHello.srp_username
+        srpUsername = clientHello.srp_username.decode("utf-8")
+        self.allegedSrpUsername = srpUsername
         #Get parameters from username
         try:
-            entry = verifierDB[clientHello.srp_username]
+            entry = verifierDB[srpUsername]
         except KeyError:
             for result in self._sendError(\
                     AlertDescription.unknown_psk_identity):
@@ -1314,7 +1376,7 @@ class TLSConnection(TLSRecordLayer):
 
         #Create ServerKeyExchange, signing it if necessary
         serverKeyExchange = ServerKeyExchange(cipherSuite)
-        serverKeyExchange.createSRP(N, g, stringToBytes(s), B)
+        serverKeyExchange.createSRP(N, g, s, B)
         if cipherSuite in CipherSuite.srpCertSuites:
             hashBytes = serverKeyExchange.hash(clientHello.random,
                                                serverHello.random)
@@ -1355,7 +1417,7 @@ class TLSConnection(TLSRecordLayer):
 
         #Calculate premaster secret
         S = powMod((A * powMod(v,u,N)) % N, b, N)
-        premasterSecret = numberToBytes(S)
+        premasterSecret = numberToByteArray(S)
         
         yield premasterSecret
 
@@ -1452,10 +1514,10 @@ class TLSConnection(TLSRecordLayer):
             if self.version == (3,0):
                 masterSecret = calcMasterSecret(self.version, premasterSecret,
                                          clientHello.random, serverHello.random)
-                verifyBytes = self._calcSSLHandshakeHash(masterSecret, "")
+                verifyBytes = self._calcSSLHandshakeHash(masterSecret, b"")
             elif self.version in ((3,1), (3,2)):
-                verifyBytes = stringToBytes(self._handshake_md5.digest() +\
-                                            self._handshake_sha.digest())
+                verifyBytes = self._handshake_md5.digest() + \
+                                self._handshake_sha.digest()
             for result in self._getMsg(ContentType.handshake,
                                       HandshakeType.certificate_verify):
                 if result in (0,1): yield result
@@ -1525,7 +1587,7 @@ class TLSConnection(TLSRecordLayer):
 
         #Calculate premaster secre
         S = powMod(dh_Yc,dh_Xs,dh_p)
-        premasterSecret = numberToBytes(S)
+        premasterSecret = numberToByteArray(S)
         
         yield premasterSecret
 
@@ -1556,7 +1618,7 @@ class TLSConnection(TLSRecordLayer):
     #*********************************************************
 
 
-    def _sendFinished(self, masterSecret):
+    def _sendFinished(self, masterSecret, nextProto=None):
         #Send ChangeCipherSpec
         for result in self._sendMsg(ChangeCipherSpec()):
             yield result
@@ -1564,6 +1626,11 @@ class TLSConnection(TLSRecordLayer):
         #Switch to pending write state
         self._changeWriteState()
 
+        if nextProto is not None:
+            nextProtoMsg = NextProtocol().create(nextProto)
+            for result in self._sendMsg(nextProtoMsg):
+                yield result
+
         #Calculate verification data
         verifyData = self._calcFinished(masterSecret, True)
         if self.fault == Fault.badFinished:
@@ -1574,7 +1641,7 @@ class TLSConnection(TLSRecordLayer):
         for result in self._sendMsg(finished):
             yield result
 
-    def _getFinished(self, masterSecret, expect_next_protocol=False):
+    def _getFinished(self, masterSecret, expect_next_protocol=False, nextProto=None):
         #Get and check ChangeCipherSpec
         for result in self._getMsg(ContentType.change_cipher_spec):
             if result in (0,1):
@@ -1589,6 +1656,7 @@ class TLSConnection(TLSRecordLayer):
         #Switch to pending read state
         self._changeReadState()
 
+        #Server Finish - Are we waiting for a next protocol echo? 
         if expect_next_protocol:
             for result in self._getMsg(ContentType.handshake, HandshakeType.next_protocol):
                 if result in (0,1):
@@ -1602,6 +1670,10 @@ class TLSConnection(TLSRecordLayer):
         else:
             self.next_proto = None
 
+        #Client Finish - Only set the next_protocol selected in the connection
+        if nextProto:
+            self.next_proto = nextProto
+
         #Calculate verification data
         verifyData = self._calcFinished(masterSecret, False)
 
@@ -1619,21 +1691,21 @@ class TLSConnection(TLSRecordLayer):
     def _calcFinished(self, masterSecret, send=True):
         if self.version == (3,0):
             if (self._client and send) or (not self._client and not send):
-                senderStr = "\x43\x4C\x4E\x54"
+                senderStr = b"\x43\x4C\x4E\x54"
             else:
-                senderStr = "\x53\x52\x56\x52"
+                senderStr = b"\x53\x52\x56\x52"
 
             verifyData = self._calcSSLHandshakeHash(masterSecret, senderStr)
             return verifyData
 
         elif self.version in ((3,1), (3,2)):
             if (self._client and send) or (not self._client and not send):
-                label = "client finished"
+                label = b"client finished"
             else:
-                label = "server finished"
+                label = b"server finished"
 
-            handshakeHashes = stringToBytes(self._handshake_md5.digest() + \
-                                            self._handshake_sha.digest())
+            handshakeHashes = self._handshake_md5.digest() + \
+                                self._handshake_sha.digest()
             verifyData = PRF(masterSecret, label, handshakeHashes, 12)
             return verifyData
         else:
@@ -1656,7 +1728,7 @@ class TLSConnection(TLSRecordLayer):
                         raise
             except GeneratorExit:
                 raise
-            except TLSAlert, alert:
+            except TLSAlert as alert:
                 if not self.fault:
                     raise
                 if alert.description not in Fault.faultAlerts[self.fault]:
